@@ -1,24 +1,22 @@
-open Stdune
+open Import
 
 type t = private
   { name : Dune_engine.Alias.Name.t
   ; recursive : bool
   ; dir : Path.Source.t
-  ; contexts : Dune_rules.Context.t list
+  ; contexts : Context.t list
   }
 
 val in_dir :
      name:Dune_engine.Alias.Name.t
   -> recursive:bool
-  -> contexts:Dune_rules.Context.t list
+  -> contexts:Context.t list
   -> Path.t
   -> t
 
 val of_string :
-     Common.t
-  -> recursive:bool
-  -> string
-  -> contexts:Dune_rules.Context.t list
-  -> t
+  Workspace_root.t -> recursive:bool -> string -> contexts:Context.t list -> t
 
 val pp : t -> _ Pp.t
+
+val request : t -> unit Action_builder.t
