@@ -14,13 +14,13 @@ val target : t -> toolchain:t -> t
 
 val equal : t -> t -> bool
 
-include Stringlike_intf.S with type t := t
+val compare : t -> t -> Ordering.t
+
+include Stringlike with type t := t
 
 module Infix : Comparator.OPS with type t = t
 
-module Map : Map.S with type key = t
-
-module Set : Set.S with type elt = t
+include Comparable_intf.S with type key := t
 
 module Top_closure :
-  Top_closure_intf.S with type key := t and type 'a monad := 'a Monad.Id.t
+  Top_closure with type key := t and type 'a monad := 'a Monad.Id.t
