@@ -1,4 +1,4 @@
-open Stdune
+open Import
 
 module Target_dir : sig
   type context_related =
@@ -9,7 +9,7 @@ module Target_dir : sig
 
   type t =
     | Install of context_related
-    | Alias of context_related
+    | Anonymous_action of context_related
     | Regular of context_related
     | Invalid of Path.Build.t
 
@@ -19,6 +19,7 @@ end
 type target_kind =
   | Regular of Context_name.t * Path.Source.t
   | Alias of Context_name.t * Path.Source.t
+  | Anonymous_action of Context_name.t
   | Install of Context_name.t * Path.Source.t
   | Other of Path.Build.t
 
@@ -39,22 +40,26 @@ val describe_target : Path.Build.t -> string
 
 val describe_path : Path.t -> string
 
-include Dune_lang.Conv.S with type t = Path.t
+include Dune_sexp.Conv.S with type t = Path.t
 
 module Local : sig
-  val encode : dir:Path.t -> Path.t Dune_lang.Encoder.t
+  val encode : dir:Path.t -> Path.t Dune_sexp.Encoder.t
 
-  val decode : dir:Path.t -> Path.t Dune_lang.Decoder.t
+  val decode : dir:Path.t -> Path.t Dune_sexp.Decoder.t
 end
 
 module Build : sig
-  include Dune_lang.Conv.S with type t = Path.Build.t
+  include Dune_sexp.Conv.S with type t = Path.Build.t
 
   val is_dev_null : t -> bool
 
   val install_dir : t
 
-  val alias_dir : t
+  val anonymous_actions_dir : t
+end
 
-  val is_alias_stamp_file : t -> bool
+module External : sig
+  val encode : Path.External.t Dune_sexp.Encoder.t
+
+  val decode : Path.External.t Dune_sexp.Decoder.t
 end
