@@ -1,5 +1,4 @@
-open! Dune_engine
-open! Stdune
+open Import
 
 module Ast = struct
   type ('k, 'v) t =
@@ -20,11 +19,11 @@ module Ast = struct
             (elt, v))
        in
        let+ clauses = repeat clause in
-       ( match clauses with
+       (match clauses with
        | [] ->
          User_error.raise ~loc
            [ Pp.text "case expression must have at least one clause" ]
-       | _ :: _ -> () );
+       | _ :: _ -> ());
        { on; clauses })
 end
 
@@ -36,5 +35,5 @@ let eval (t : _ t) ~f =
   let elem = f t.on in
   List.find_map t.clauses ~f:(fun (keys, v) ->
       Option.some_if
-        (Predicate_lang.Glob.exec keys ~standard:Predicate_lang.empty elem)
+        (Predicate_lang.Glob.test keys ~standard:Predicate_lang.false_ elem)
         v)
