@@ -1,7 +1,7 @@
-open! Dune_engine
-open Stdune
+open Import
 
-val compile_info : scope:Scope.t -> Dune_file.Executables.t -> Lib.Compile.t
+val compile_info :
+  scope:Scope.t -> Dune_file.Executables.t -> Lib.Compile.t Memo.t
 
 val rules :
      sctx:Super_context.t
@@ -10,4 +10,4 @@ val rules :
   -> scope:Scope.t
   -> expander:Expander.t
   -> Dune_file.Executables.t
-  -> Compilation_context.t * Merlin.t
+  -> (Compilation_context.t * Merlin.t) Memo.t
