@@ -1,4 +1,3 @@
-open! Dune_engine
 open Import
 
 (** This module loads and validates foreign sources from directories. *)
@@ -14,8 +13,8 @@ val for_archive : t -> archive_name:Foreign.Archive.Name.t -> Foreign.Sources.t
 val for_exes : t -> first_exe:string -> Foreign.Sources.t
 
 val make :
-     Stanza.t list Dir_with_dune.t
-  -> include_subdirs:Loc.t * Dune_file.Include_subdirs.t
+     Stanza.t list
+  -> dune_version:Syntax.Version.t
   -> lib_config:Lib_config.t
-  -> dirs:(Path.Build.t * 'a * String.Set.t) list
+  -> dirs:(Path.Build.t * 'a * Filename.Set.t) list
   -> t
