@@ -1,5 +1,4 @@
-open! Dune_engine
-open Stdune
+open Import
 
 module Select : sig
   module Choice : sig
@@ -24,6 +23,8 @@ type t =
   | Re_export of (Loc.t * Lib_name.t)
   | Select of Select.t
 
+val equal : t -> t -> bool
+
 val to_dyn : t -> Dyn.t
 
 val direct : Loc.t * Lib_name.t -> t
@@ -33,5 +34,13 @@ val re_export : Loc.t * Lib_name.t -> t
 val decode : allow_re_export:bool -> t Dune_lang.Decoder.t
 
 module L : sig
-  val field_encode : t list -> name:string -> Dune_lang.Encoder.field
+  type nonrec t = t list
+
+  val field_encode : t -> name:string -> Dune_lang.Encoder.field
+
+  val decode :
+       allow_re_export:bool
+    -> (t, Dune_lang.Decoder.values) Dune_lang.Decoder.parser
+
+  val of_pps : Lib_name.t list -> t
 end
