@@ -1,7 +1,6 @@
 (** OCaml flags *)
-open! Dune_engine
 
-open! Stdune
+open! Import
 
 type t
 
@@ -20,8 +19,17 @@ val make :
   -> default:t
   -> eval:
        (   Ordered_set_lang.Unexpanded.t
-        -> standard:string list Build.t
-        -> string list Build.t)
+        -> standard:string list Action_builder.t
+        -> string list Action_builder.t)
+  -> t
+
+val make_with_melange :
+     melange:Ordered_set_lang.Unexpanded.t
+  -> default:t
+  -> eval:
+       (   Ordered_set_lang.Unexpanded.t
+        -> standard:string list Action_builder.t
+        -> string list Action_builder.t)
   -> t
 
 val default : dune_version:Dune_lang.Syntax.Version.t -> profile:Profile.t -> t
@@ -30,14 +38,12 @@ val empty : t
 
 val of_list : string list -> t
 
-val get : t -> Mode.t -> string list Build.t
+val get : t -> Lib_mode.t -> string list Action_builder.t
 
 val append_common : t -> string list -> t
 
-val prepend_common : string list -> t -> t
-
 val with_vendored_warnings : t -> t
 
-val common : t -> string list Build.t
+val with_vendored_alerts : t -> t
 
-val dump : t -> Dune_lang.t list Build.t
+val dump : t -> Dune_lang.t list Action_builder.t
