@@ -1,14 +1,16 @@
 (** Preprocessing of OCaml source files *)
-open! Dune_engine
 
-open! Stdune
-open! Import
+open Import
+
+val pped_modules_map :
+     Preprocess.Without_instrumentation.t Preprocess.t Module_name.Per_item.t
+  -> Ocaml.Version.t
+  -> (Module.t -> Module.t) Staged.t
 
 val make :
      Super_context.t
   -> dir:Path.Build.t
   -> expander:Expander.t
-  -> dep_kind:Lib_deps_info.Kind.t
   -> lint:Preprocess.Without_instrumentation.t Preprocess.Per_module.t
   -> preprocess:Preprocess.Without_instrumentation.t Preprocess.Per_module.t
   -> preprocessor_deps:Dep_conf.t list
@@ -26,20 +28,18 @@ val get_ppx_driver :
   -> lib_name:Lib_name.Local.t option
   -> flags:String_with_vars.t list
   -> (Loc.t * Lib_name.t) list
-  -> (Path.Build.t * string list) Or_exn.t
+  -> (Path.Build.t * string list) Action_builder.t
 
-val gen_rules : Super_context.t -> string list -> unit
+val gen_rules : Super_context.t -> string list -> unit Memo.t
 
-val chdir : Action_unexpanded.t -> Action_unexpanded.t
-
-val action_for_pp :
-     dep_kind:Lib_deps_info.Kind.t
+val action_for_pp_with_target :
+     sandbox:Sandbox_config.t
   -> loc:Loc.t
   -> expander:Expander.t
   -> action:Action_unexpanded.t
   -> src:Path.Build.t
-  -> target:Path.Build.t option
-  -> Action.t Build.With_targets.t
+  -> target:Path.Build.t
+  -> Action.Full.t Action_builder.With_targets.t
 
 val ppx_exe :
-  Super_context.t -> scope:Scope.t -> Lib_name.t -> Path.Build.t Or_exn.t
+  Context.t -> scope:Scope.t -> Lib_name.t -> Path.Build.t Resolve.Memo.t
