@@ -1,5 +1,4 @@
-open! Dune_engine
-open Stdune
+open Import
 
 module Source : sig
   type t
@@ -8,14 +7,14 @@ module Source : sig
 
   val loc : t -> Loc.t
 
-  val modules : t -> Pp_spec.t -> Modules.t
+  val modules : t -> Pp_spec.t -> Modules.t Memo.t
 
   val obj_dir : t -> Path.Build.t Obj_dir.t
 end
 
 type t
 
-val setup_rules : t -> unit
+val setup_rules : t -> unit Memo.t
 
 val make :
      cctx:Compilation_context.t
@@ -23,13 +22,21 @@ val make :
   -> preprocess:Preprocess.Without_instrumentation.t Preprocess.t
   -> t
 
-val print_toplevel_init_file :
-  include_paths:Path.Set.t -> files_to_load:Path.t list -> unit
+type directives =
+  { include_paths : Path.Set.t
+  ; files_to_load : Path.t list
+  ; uses : Path.t list
+  ; pp : string option
+  ; ppx : string option
+  ; code : string list
+  }
+
+val print_toplevel_init_file : directives -> unit
 
 module Stanza : sig
   val setup :
        sctx:Super_context.t
     -> dir:Path.Build.t
     -> toplevel:Dune_file.Toplevel.t
-    -> unit
+    -> unit Memo.t
 end
