@@ -1,5 +1,3 @@
-open Stdune
-
 module type Input = sig
   type t
 
@@ -18,6 +16,8 @@ module type S = sig
   val set : 'a t -> key -> 'a -> unit
 
   val find : 'a t -> key -> 'a option
+
+  val iter : 'a t -> f:('a -> unit) -> unit
 end
 
 module type Instance = sig
@@ -33,5 +33,7 @@ module type Instance = sig
 
   val find : t -> key -> value option
 
+  val iter : t -> f:(value -> unit) -> unit
+
   val store : t
 end
